uniform sampler3D densityTex;
uniform sampler2D backfaceTex;
uniform sampler2D backfaceObjPosTex;
uniform sampler2D histogramTex;
uniform sampler2D noiseTex;
uniform float noiseOn;
uniform float startSlice;
uniform float minDepth;
uniform float maxDepth;
uniform float sampleRate;
uniform float errorEpsilon;
varying vec4 pos;
varying vec4 objPos;
uniform mat4 toShadowmap;

void main(void)
 {
	/*float sampleRate=200.0;
	float errorEpsilon=0.05;*/
 
	vec4 endPos=texture2D(backfaceTex, vec2(gl_FragCoord.x/2048.0, gl_FragCoord.y/2048.0)).rgba;
	vec4 endObjPos=texture2D(backfaceObjPosTex, vec2(gl_FragCoord.x/2048.0, gl_FragCoord.y/2048.0)).rgba;
	vec3 startPos=gl_TexCoord[0].xyz;
	vec3 totalTraceDir=endPos.rgb-startPos;
	float traceLength=length(totalTraceDir);
	vec3 traceDir=normalize(totalTraceDir)/sampleRate;		// stepsize muss vom datensatz abhaengen
	float stepLength=length(traceDir);
	vec4 color=vec4(0.0,0.0,0.0,1.0);
	
	vec3 objPosDir=(endObjPos.xyz-objPos.xyz)/(totalTraceDir/traceDir);
	vec4 curObjPos=objPos.xyzw;
	
	float accAlpha=0.0;
	
	vec3 noise=(texture2D(noiseTex, vec2(gl_FragCoord.x/128.0, gl_FragCoord.y/128.0)).a)*traceDir*noiseOn;
	
	float currentLength=0.0;
	vec3 currentPos=startPos-noise;
	vec4 currentColor;
	
	float curDensity=0.0;
	
	float startDepth=-(pos.z-maxDepth)/(maxDepth-minDepth);
	float currentDepth=startDepth;
	
	float currentAlphaSlopeUpper=0.0;
	float currentAlphaSlopeLower=0.0;
	float tempAlphaSlopeUpper=0.0;
	float tempAlphaSlopeLower=0.0;
	
	bool firstSlope=true;
	
	float currentAlphaStart=0.0;
	
	float depthToStore=0.0;
	float alphaToStore=0.0;
	
	float iterations=0.0;
	
	bool breakFound=false;
	
	for (int i=0; i<8; i++) {
		iterations=0.0;
		breakFound=false;
		while (currentLength<traceLength) {// && accAlpha<0.9) {
			curDensity=texture3D(densityTex, currentPos).r;
			currentColor=vec4(texture2D(histogramTex, vec2(curDensity, 0.75)));
		
			iterations+=1.0;
			
			depthToStore=currentDepth;
			
			vec4 shadowPos=toShadowmap*curObjPos;
			currentDepth=shadowPos.z/shadowPos.w;
			
			currentPos+=traceDir;
			currentLength+=stepLength;
			
			curObjPos.xyz+=objPosDir;
			
			alphaToStore=accAlpha;


			accAlpha+=(1.0-accAlpha)*currentColor.a;
			
			if (accAlpha>0.0) {
				if (i==0) {
					alphaToStore=0.0;
					currentAlphaStart=0.0;
				}
				
				tempAlphaSlopeUpper=((accAlpha+errorEpsilon)-currentAlphaStart)/iterations;
				tempAlphaSlopeLower=((accAlpha-errorEpsilon)-currentAlphaStart)/iterations;
				
				if (firstSlope) {
					currentAlphaSlopeUpper=tempAlphaSlopeUpper;
					currentAlphaSlopeLower=tempAlphaSlopeLower;
					firstSlope=false;
				}
				
				if (tempAlphaSlopeUpper<currentAlphaSlopeLower || tempAlphaSlopeLower>currentAlphaSlopeUpper) {
					currentAlphaStart=alphaToStore;
					firstSlope=true;
					breakFound=true;
					break;
				} else {
					currentAlphaSlopeUpper=min(currentAlphaSlopeUpper,tempAlphaSlopeUpper);
					currentAlphaSlopeLower=max(currentAlphaSlopeLower,tempAlphaSlopeLower);
				}
			}
		}
		
		if (!breakFound) {
			depthToStore=1.0;
			alphaToStore=accAlpha;
		}
		
		if (i%2==0) {
			gl_FragData[i/2].rg=vec2(depthToStore,1.0-alphaToStore);
		} else {
			gl_FragData[i/2].ba=vec2(depthToStore,1.0-alphaToStore);
		}
	}
 }
