uniform sampler3D densityTex;
uniform sampler2D backfaceTex;
uniform sampler2D backfaceObjPosTex;
uniform sampler2D histogramTex;
uniform sampler2D noiseTex;
uniform sampler2D dsm12;
uniform sampler2D dsm34;
uniform sampler2D dsm56;
uniform sampler2D dsm78;
uniform float noiseOn;
uniform float startSlice;
uniform float minDepth;
uniform float maxDepth;
uniform mat4 toShadowmap;
uniform float sampleRate;
uniform float bias;
uniform vec3 lightPos;
varying vec4 pos;
varying vec4 objPos;

void main(void)
 {
	//float sampleRate=300.0;
 
	vec4 endPos=texture2D(backfaceTex, vec2(gl_FragCoord.x/2048.0, gl_FragCoord.y/2048.0)).rgba;
	vec4 endObjPos=texture2D(backfaceObjPosTex, vec2(gl_FragCoord.x/2048.0, gl_FragCoord.y/2048.0)).rgba;
	vec3 startPos=gl_TexCoord[0].xyz;
	vec3 totalTraceDir=endPos.rgb-startPos;
	float traceLength=length(totalTraceDir);
	vec3 traceDir=normalize(totalTraceDir)/sampleRate;
	float stepLength=length(traceDir);
	vec4 color=vec4(0.0,0.0,0.0,1.0);

	vec3 objPosDir=(endObjPos.xyz-objPos.xyz)/(totalTraceDir/traceDir);
	
	float accAlpha=0.0;
	
	vec3 noise=(texture2D(noiseTex, vec2(gl_FragCoord.x/128.0, gl_FragCoord.y/128.0)).a)*traceDir*noiseOn;
	
	float currentLength=0.0;
	vec3 currentPos=startPos-noise;
	vec4 currentColor;
	
	vec4 curObjPos=objPos.xyzw;
	
	float curDensity=0.0;
	vec3 curNormal=vec3(1.0,0.0,0.0);
	
	vec3 lightVec=normalize(lightPos);
	vec3 halfVec=vec3(1.0,0.0,0.0);
	
	float ambientFactor=0.2;
	float diffuse=0.0;
	float shininess=100.0;
	float specularFactor=2.0;
	
	float specular=0.0;
	
	vec3 viewVec=normalize(-pos.xyz);
	
	float startDepth=-(pos.z-maxDepth)/(maxDepth-minDepth);
	float currentDepth=startDepth;
	
	while (currentLength<traceLength && accAlpha<0.9) {
		
		curDensity=texture3D(densityTex, currentPos).r;
		currentColor=vec4(texture2D(histogramTex, vec2(curDensity, 0.75)));
		
		if (currentDepth>startSlice && currentColor.a>0.01) {
		
			curNormal=normalize(texture3D(densityTex, currentPos).gba*2.0-1.0);	
			curNormal=gl_NormalMatrix*curNormal;
		
			diffuse=max(0.0,dot(curNormal,lightVec));
			halfVec=reflect(-lightVec,curNormal);
			specular=specularFactor*pow(max(0.0, dot(halfVec, viewVec)),shininess);
			
			currentColor.rgb=currentColor.rgb*diffuse+currentColor.rgb*ambientFactor;
			currentColor.rgb+=specular;
			currentColor.a=currentColor.a/sampleRate*256.0;
		
		
				
			vec4 shadowPos=toShadowmap*curObjPos;
			shadowPos.x=(shadowPos.x/shadowPos.w+1.0)*0.5;
			shadowPos.y=(shadowPos.y/shadowPos.w+1.0)*0.5;
			
			shadowPos.z=shadowPos.z/shadowPos.w-bias;//0.05;
			
			
			vec4 depthAlpha[4];
			
			depthAlpha[0]=texture2D(dsm12,shadowPos.xy).rgba;
			depthAlpha[1]=texture2D(dsm34,shadowPos.xy).rgba;
			depthAlpha[2]=texture2D(dsm56,shadowPos.xy).rgba;
			depthAlpha[3]=texture2D(dsm78,shadowPos.xy).rgba;
			
			
			float deltaDepth=max(0.0,shadowPos.z-depthAlpha[0].r);
			float depthRange=depthAlpha[0].b-depthAlpha[0].r;
			
			float firstAlpha=depthAlpha[0].g;
			float secondAlpha=depthAlpha[0].a;
			
			
			float depthDiff=0.0;
			
			if (shadowPos.z>depthAlpha[0].b)
			{
				depthDiff=shadowPos.z-depthAlpha[0].b;
				if (deltaDepth>depthDiff) {
					deltaDepth=depthDiff;
					firstAlpha=depthAlpha[0].a;
					depthRange=depthAlpha[1].r-depthAlpha[0].b;
					secondAlpha=depthAlpha[1].g;
				}
			}
			
			for (int i=3; i>0; i--) {
				if (shadowPos.z>depthAlpha[i].r)
				{
					depthDiff=shadowPos.z-depthAlpha[i].r;
					deltaDepth=depthDiff;
					depthRange=depthAlpha[i].b-depthAlpha[i].r;
					firstAlpha=depthAlpha[i].g;
					secondAlpha=depthAlpha[i].a;
					break;
				}
				if (shadowPos.z>depthAlpha[i].b)
				{
					depthDiff=shadowPos.z-depthAlpha[i].b;
					deltaDepth=depthDiff;
					firstAlpha=depthAlpha[i].a;
					if (i<3) {
						depthRange=depthAlpha[i+1].r-depthAlpha[i].b;
						secondAlpha=depthAlpha[i+1].g;
					} else {
						depthRange=deltaDepth;
						secondAlpha=depthAlpha[i].a;
					}
					break;
				}
			}
			
			if (depthRange>0.0) {
				deltaDepth/=depthRange;
			}
			
			currentColor.rgb*=max(ambientFactor,firstAlpha+deltaDepth*(secondAlpha-firstAlpha));
			
			color = vec4(color.rgb+(1.0-accAlpha)*currentColor.rgb*currentColor.a,1.0);
		
			accAlpha+=(1.0-accAlpha)*currentColor.a;
		}
		
		curObjPos.xyz+=objPosDir;
	
		currentPos+=traceDir;
		currentLength+=stepLength;
		
		currentDepth=startDepth+currentLength/traceLength*(endPos.a-startDepth);
	}

	gl_FragColor=color;
	
	 
 }
