uniform sampler3D densityTex;
uniform sampler2D backfaceTex;
uniform sampler2D histogramTex;
uniform sampler2D noiseTex;
uniform float noiseOn;
uniform float startSlice;
uniform float minDepth;
uniform float maxDepth;
uniform float sampleRate;
uniform vec3 lightPos;
varying vec4 pos;

void main(void)
 {
	//float sampleRate=300.0;//startSampleRate;
	
	float alphaPow=0.0;
 
	vec4 endPos=texture2D(backfaceTex, vec2(gl_FragCoord.x/2048.0, gl_FragCoord.y/2048.0)).rgba;
	vec3 startPos=gl_TexCoord[0].xyz;
	vec3 totalTraceDir=endPos.rgb-startPos;
	float traceLength=length(totalTraceDir);
	vec3 traceDir=normalize(totalTraceDir)/sampleRate;		// stepsize muss vom datensatz abhaengen
	float stepLength=length(traceDir);
	vec4 color=vec4(0.0,0.0,0.0,1.0);
	
	float accAlpha=0.0;
	
	vec3 noise=(texture2D(noiseTex, vec2(gl_FragCoord.x/128.0, gl_FragCoord.y/128.0)).a)*traceDir*noiseOn;
	
	float currentLength=0.0;
	vec3 currentPos=startPos-noise;
	vec4 currentColor;
	
	float maxDensity=0.0;
	float curDensity=0.0;
	vec3 curNormal=vec3(1.0,0.0,0.0);
	
	vec3 lightVec=normalize(lightPos);
	vec3 halfVec=vec3(1.0,0.0,0.0);
	
	float ambientFactor=0.2;
	float diffuse=0.0;
	float shininess=100.0;
	float specularFactor=2.0;
	
	float specular=0.0;
	float borderline=0.0;
	
	vec3 viewVec=normalize(-pos.xyz);
	
	vec4 curPos=pos.xyzw;
	
	float startDepth=-(pos.z-maxDepth)/(maxDepth-minDepth);
	float currentDepth=startDepth;
	
	while (currentLength<traceLength && accAlpha<0.9) {
		
		curDensity=texture3D(densityTex, currentPos).r;
		currentColor=vec4(texture2D(histogramTex, vec2(curDensity, 0.75)));
		
		if (currentDepth>startSlice && currentColor.a>0.01) {
			
		
			curNormal=normalize(texture3D(densityTex, currentPos).gba*2.0-1.0);	
			curNormal=gl_NormalMatrix*curNormal;
		
			diffuse=max(0.0,dot(curNormal,lightVec));
			halfVec=reflect(-lightVec,curNormal);
			specular=specularFactor*pow(max(0.0, dot(halfVec, viewVec)),shininess);
			
			currentColor.rgb=currentColor.rgb*diffuse+currentColor.rgb*ambientFactor;
			currentColor.rgb+=specular;
			currentColor.a=currentColor.a/sampleRate*256.0;
			
			
			color = vec4(color.rgb+(1.0-accAlpha)*currentColor.rgb*currentColor.a,1.0);
		
			accAlpha+=(1.0-accAlpha)*currentColor.a;
		}
	
		currentPos+=traceDir;
		currentLength+=stepLength;
		
		currentDepth=startDepth+currentLength/traceLength*(endPos.a-startDepth);
	}

	gl_FragColor=color;
	 
 }
